module sseg_multiplexing
#(
    parameter    N = 18 // counter
) (
    input            clk, rstb,
    input      [3:0] in0, in1, in2, in3,
    input      [3:0] dp_in,
    output reg [3:0] an,
    output reg [7:0] sseg
);

reg  [N-1:0] q_reg;
wire [N-1:0] q_next;
reg  [3:0]   in;
reg          dp;

// counter
always @(posedge clk or negedge rstb) begin
    if (~rstb) q_reg <= 0;
    else       q_reg <= q_next;
end

// next-state logic
assign q_next = q_reg+1;

// 4-to-1 multiplexer
always @* begin
    case (q_reg[N-1:N-2])
        2'b00: begin
            an = 4'b1110;
            in = in0;
            dp = dp_in[0];
        end
        2'b01: begin
            an = 4'b1101;
            in = in1;
            dp = dp_in[1];
        end
        2'b10: begin
            an = 4'b1011;
            in = in2;
            dp = dp_in[2];
        end
        default: begin
            an = 4'b0111;
            in = in3;
            dp = dp_in[3];
        end
    endcase
end

// sseg decoder
always @* begin
    case (in)
        4'h0: sseg[6:0] = 7'b000_0001;
        4'h1: sseg[6:0] = 7'b100_1111;
        4'h2: sseg[6:0] = 7'b001_0010;
        4'h3: sseg[6:0] = 7'b000_0110;
        4'h4: sseg[6:0] = 7'b100_1100;
        4'h5: sseg[6:0] = 7'b010_0100;
        4'h6: sseg[6:0] = 7'b010_0000;
        4'h7: sseg[6:0] = 7'b000_1111;
        4'h8: sseg[6:0] = 7'b000_0000;
        4'h9: sseg[6:0] = 7'b000_0100;
        4'ha: sseg[6:0] = 7'b000_1000;
        4'hb: sseg[6:0] = 7'b110_0000;
        4'hc: sseg[6:0] = 7'b011_0001;
        4'hd: sseg[6:0] = 7'b100_0010;
        4'he: sseg[6:0] = 7'b011_0000;
        default: sseg[6:0] = 7'b011_1000; // 4'hf
    endcase
    sseg[7] = dp;
end

endmodule