module universal_counter
#(
   parameter      N = 4
) (
   input          clk, rstb,
   input          sclr, load, en, up,
   input  [N-1:0] d,
   output [N-1:0] q,
   output         max_tick, min_tick
);

reg [N-1:0] state_reg;
reg [N-1:0] state_next;

// state register
always @(posedge clk or negedge rstb) begin
   if (~rstb) state_reg <= 0;
   else       state_reg <= state_next;
end

// next-state logic
always @* begin
   if (sclr)        state_next = 0;
   else if (load)   state_next = d;
   else if (en&up)  state_next = state_reg + 1;
   else if (en&~up) state_next = state_reg - 1;
   else             state_next = state_reg;
end

// output logic
assign q = state_reg;
assign max_tick = &state_reg;
assign min_tick = |state_reg;

endmodule